
import os
from typing import Callable

import wrapt

from autoboot.applications import AutoBoot
from autoboot.annotation.component import component
from autoboot.process import get_yml_value


def env(keypath: str) -> str | None:
  """get env value from keypath, support find in `.env` and `.yml`."""
  
  if "." in keypath:
    value = get_yml_value(AutoBoot.get_config_data(), keypath=keypath)
    if not value:
      return None
    return str(value)
  else:
    value = os.getenv(keypath)
    if value:
      return value
    value = AutoBoot.get_config_data().get(keypath)
    return str(value) if value else None
    

def value(keypath: str) -> Callable[..., str | None]:
  """get value from config file."""
  
  @wrapt.decorator
  def decorator(fn, instance, args, kwargs) -> str | None:
    return env(keypath) or fn(*args, **kwargs)
  return decorator
      

def value_component(keypath: str) -> Callable[..., str | None]:
  """get value from config file and cache as component."""
  
  @wrapt.decorator
  @component(name=f"env[{keypath}]")
  def decorator(fn, instance, args, kwargs) -> str | None:
    return env(keypath) or fn(*args, **kwargs)
  return decorator